/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */
 
package estella.lib;
import  estella.lib.*;
import  estella.gui.*;

import  java.util.List;
import  java.util.LinkedList;
import  java.util.Collections;
import  java.util.Iterator;
import  java.util.Vector;

/**
* Gerencia uma lista do tipo Servent
* Essa classe depende da classe de Servent, no qual gerencia uma Lista de objetos
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 01/12/2005
* @see Lista
* @see Servent
*/
public class ServentLista extends Lista
{
 
	public ServentLista()
	{
		super();
	}
	
  public ServentLista(List LISTA)
  {
    super(LISTA);
  }
  
  /**
	* Verifica se existe um host (ip / nome computador) na lista
	* @param host a ser verificado
	* @return true ou false
	*/
  public boolean existe(String host)
  {
    Iterator i;
    try {
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {    
          Servent servent = (Servent) i.next();
					Conexao conexao = servent.conexao;
          if(servent.host.equals(host)) {
            return true;
					}
        }
      }
      return false;
    }
    catch(Exception FalhaConsultaObj) {
      LOG.info(" falha ao consultar se o host existe na lista " + FalhaConsultaObj.getMessage());
      return false;
    }
  }

  /**
	* Verifica se existe um host (ip / nome computador) na lista e o atualiza com os novos dados
	* @param host a ser verificado
	* @return true ou false
	*/
  public boolean atualiza(Servent novoServent)
  {
    Iterator i;
		
    try {
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {    
          Servent servent = (Servent) i.next();
					Conexao conexao = servent.conexao;
          if(servent.host.equals(novoServent.host)) {
						remove(servent);
						return adiciona(novoServent);
					}
        }
      }
      return false;
    }
    catch(Exception FalhaConsultaObj) {
      LOG.info(" falha ao consultar se o host existe na lista " + FalhaConsultaObj.getMessage());
      return false;
    }
  }
	
  /**
	* Verifica se existe um host (ip / nome computador) na lista e o atualiza com os novos dados
	* @param host a ser verificado
	* @return true ou false
	*/
  public boolean atualiza(String host, int numArqComp, int tamArqComp, String ultVerificacao)
  {
    Iterator i;
		boolean atualiza=false;
		
    try {
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {    
          Servent servent = (Servent) i.next();
					Conexao conexao = servent.conexao;
          if(servent.host.equals(host)) {
						Servent novoServent = new Servent(conexao, numArqComp, tamArqComp, ultVerificacao);
						adiciona(novoServent);
						if(remove(host, conexao.porta()) != null)
							return true;
					}
        }
      }
      return false;
    }
    catch(Exception FalhaConsultaObj) {
      LOG.info(" falha ao consultar se o host existe na lista " + FalhaConsultaObj.getMessage());
      return false;
    }
  }
	
	/**
	* Verifica se existe um host (ip / nome computador) na lista
	* @param host a ser verificado
	* @return true ou false
	*/
  public Servent remove(String host, int porta)
  {
    Iterator i;
		
    try {
       i = LISTA.iterator();
       while (i.hasNext()) {    
					Servent servent = (Servent) i.next();
          if(servent.host.equals(host))
						if(servent.porta == porta) {
							remove(servent);
							return servent;
						}
        }
     return null;
    }
    catch(Exception FalhaConsultaObj) {
      LOG.falha(" ao remover host da lista -> host=" + host + ":" + porta + " tamanho da lista=" + tamanho() + FalhaConsultaObj.getMessage());
      return null;
    }
  }
	
	/**
	* Envia a mensagem para todos da lista
	* @param msg String com a mensagem
	* @return true ou false
	*/
	public boolean envia(String msg)
  {
    Iterator i;
    try {
			synchronized(LISTA) {
				i = LISTA.iterator();
				while (i.hasNext()) {
					Servent servent = (Servent) i.next();
					Conexao conexao = servent.conexao;
					if(conexao.status().isConnected()) {
						if(!conexao.status().getInetAddress().getHostAddress().equals(Dados.ip))
						if(!conexao.envia(msg)) {
							LOG.falha(" ao enviar mensagem para : " + conexao.status());
						}							
					} else {
						conexao.fecha();
						remove(servent);
					}
				}
			}
      return true;
		}
    catch(Exception FalhaConsultaObj) {
      LOG.info(" ao enviar mensagem " + FalhaConsultaObj.getMessage());
      return false;
    }
  }
	
	/**
	* Envia a mensagem para todos da lista menos para o host de origem
	* @param hostOrigem string com o host que nao eh pra enviar
	* @param msg String com a mensagem
	* @return true ou false
	*/
	public boolean envia(String msg, String hostOrigem)
  {
    Iterator i;
    try {
			synchronized(LISTA) {
				i = LISTA.iterator();
				while (i.hasNext()) {
					Servent servent = (Servent) i.next();
					Conexao conexao = servent.conexao;
					if(conexao.status().isConnected()) {
						if(!conexao.status().getInetAddress().getHostAddress().equals(Dados.ip))
						if(!conexao.status().getInetAddress().getHostAddress().equals(hostOrigem))
						if(!conexao.envia(msg)) {
							conexao.fecha();
							remove(servent);
						}							
					} else {
						conexao.fecha();
						remove(servent);
					}
				}
			}
      return true;
		}
    catch(Exception FalhaConsultaObj) {
      LOG.info(FalhaConsultaObj.getMessage());
      return false;
    }
  }
	
  public boolean fechaConexoes()
  {
    Iterator i;
    try {
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {
					Servent servent = (Servent) i.next();
					Conexao conexao = servent.conexao;
          conexao.fecha();
				}
				return true;
      }
    }
    catch(Exception FalhaConsultaObj) {
      LOG.info(FalhaConsultaObj.getMessage());
      return false;
    }
  }
	
	/**
	* Atualiza tabela de servents online na interface grafica
	* @return boolean true ou false
	*/
	public boolean atualizaTabelaServentsOnline()
	{
		Iterator i;
    try {
			synchronized(JanelaPrincipal.linhaTabelaServentsOnline) {
				JanelaPrincipal.linhaTabelaServentsOnline.clear();
			}
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {
					Servent servent = (Servent) i.next();
					Conexao conexao = servent.conexao;
						Vector serv = new Vector();
						serv.addElement(conexao.host());
						serv.addElement(conexao.porta() + "");
						serv.addElement(servent.numArqComp + "");
						serv.addElement(servent.tamArqComp + "");
					synchronized(JanelaPrincipal.linhaTabelaServentsOnline) {
						JanelaPrincipal.linhaTabelaServentsOnline.addElement(serv);
					}
				}
			}
			synchronized(JanelaPrincipal.tabelaServentsOnline) {
				JanelaPrincipal.tabelaServentsOnline.removeNotify();
				JanelaPrincipal.tabelaServentsOnline.addNotify();
			}
			synchronized(JanelaPrincipal.labelServentsOnline) {
				JanelaPrincipal.labelServentsOnline.setText(Dados.onlineServents.tamanho() + "");
			}
			synchronized(JanelaPrincipal.labelOnline) {
				if(Dados.onlineServents.tamanho() > 0)
					JanelaPrincipal.labelOnline.setText("sim");
				else
					JanelaPrincipal.labelOnline.setText("no");
			}
			return true;
		}
    catch(Exception FalhaConsultaObj) {
      LOG.info(FalhaConsultaObj.getMessage());
      return false;
    }
	}
	
	/**
	* Atualiza tabela de servents offline  na interface grafica
	* @return boolean true ou false
	*/
	public boolean atualizaTabelaServents()
	{
		Iterator i;
    try {
			synchronized(JanelaPrincipal.linhaTabelaServents) {
				JanelaPrincipal.linhaTabelaServents.clear();
			}
      synchronized(LISTA) {
        i = LISTA.iterator();
        while (i.hasNext()) {
					Servent servent = (Servent) i.next();
					Conexao conexao = servent.conexao;
						Vector serv = new Vector();
						serv.addElement(servent.host);
						serv.addElement(servent.porta + "");
						serv.addElement(servent.numArqComp + "");
						serv.addElement(servent.tamArqComp + "");
					synchronized(JanelaPrincipal.linhaTabelaServents) {
						JanelaPrincipal.linhaTabelaServents.addElement(serv);
					}
				}
			}
			synchronized(JanelaPrincipal.tabelaServents) {
				JanelaPrincipal.tabelaServents.removeNotify();
				JanelaPrincipal.tabelaServents.addNotify();
			}
			return true;
    }
    catch(Exception FalhaConsultaObj) {
      LOG.info(FalhaConsultaObj.getMessage());
      return false;
    }
	}
}
